<?php

namespace Drupal\tome_aws;

use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Extension\ModuleHandler;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\State\StateInterface;
use Drupal\tome_base\PathTrait;
use Drupal\tome_static\StaticGeneratorInterface;

/**
 * A service to wrap batch operations for deploying to AWS.
 */
class DeployBatch {

  use PathTrait;
  use DependencySerializationTrait;

  /**
   * The static generator.
   *
   * @var \Drupal\tome_static\StaticGeneratorInterface
   */
  protected $static;

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManager
   */
  protected $entityTypeManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandler
   */
  protected $moduleHandler;

  /**
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * @var \Drupal\tome_aws\RemoteContentMetadata
   */
  protected $remoteContentMetadata;

  /**
   * @var \Drupal\tome_aws\AwsS3Client
   */
  protected $awsS3Client;

  /**
   * DeployBatch constructor.
   *
   * @param \Drupal\tome_static\StaticGeneratorInterface $static
   *   The static generator.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   * @param \Drupal\Core\Entity\EntityTypeManager $entity_type_manager
   * @param \Drupal\Core\Extension\ModuleHandler $module_handler
   * @param \Drupal\Core\State\StateInterface $state
   * @param \Drupal\tome_aws\RemoteContentMetadata $remote_content_metadata
   * @param \Drupal\tome_aws\AwsS3Client $aws_s3_client
   */
  public function __construct(StaticGeneratorInterface $static, ConfigFactoryInterface $config_factory, FileSystemInterface $file_system, EntityTypeManager $entity_type_manager, ModuleHandler $module_handler, StateInterface $state, RemoteContentMetadata $remote_content_metadata, AwsS3Client $aws_s3_client) {

    $this->static                = $static;
    $this->config                = $config_factory->get('tome_aws.settings');
    $this->fileSystem            = $file_system;
    $this->entityTypeManager     = $entity_type_manager;
    $this->moduleHandler         = $module_handler;
    $this->state                 = $state;
    $this->remoteContentMetadata = $remote_content_metadata;
    $this->awsS3Client           = $aws_s3_client;
  }

  /**
   * Determines if the site is configured to properly deploy.
   *
   * @return bool
   *   Whether or not the site is configured to properly deploy.
   */
  public function checkConfiguration() {

    return !empty($this->config->get('aws_s3_client_id')) && !empty($this->config->get('aws_s3_client_secret')) && !empty($this->config->get('aws_s3_bucket_name')) && !empty($this->config->get('aws_s3_bucket_prefix'));
  }

  /**
   * Determines if a static build exists.
   *
   * @return bool
   *   Whether or not a static build exists.
   */
  public function checkStaticBuild() {

    return file_exists($this->static->getStaticDirectory());
  }

  public function getGeneratedBaseURL() {

    return $this->state->get(StaticGeneratorInterface::STATE_KEY_URL, 'http://127.0.0.1');
  }

  /**
   * Gets a batch object for deploying to AWS.
   *
   * @return \Drupal\Core\Batch\BatchBuilder
   *   The batch builder.
   */
  public function getBatch() {

    $batch_builder = new BatchBuilder();

    // 1. Cache remote content metadata
    $this->remoteContentMetadata->sync();

    // 2. Incremental content Deploy
    $files = $this->getFilesToUpload();
    foreach ($files as $file) {
      $batch_builder->addOperation([$this->awsS3Client, 'uploadFile'], [$file, $this->static->getStaticDirectory()]);
    }

    // 3. Set remote redirects
    foreach ($this->getLocalRedirects() as $source => $destination) {
      $batch_builder->addOperation([$this->awsS3Client, 'setRemoteRedirect'], [
        $source,
        $destination
      ]);
    }

    // todo: 4. Delete obsolete files.

    return $batch_builder;

  }

  public function getFilesToUpload() {
    $files = [];
    foreach ($this->fileSystem->scanDirectory($this->static->getStaticDirectory(), '/.*/') as $file) {
      $relativeFilePath = $this->awsS3Client->getRemoteFilePath($file->uri, $this->static->getStaticDirectory());
      $remoteETagCache  = $this->remoteContentMetadata->get($relativeFilePath);
      $remoteETag       = $remoteETagCache ? $remoteETagCache->data : NULL;
      $localETag        = md5_file($file->uri);
      if ($remoteETag !== $localETag) {
        $files[] = $file->uri;
      }
    }
    return $files;
  }

  public function getLocalRedirects() {

    if (!$this->moduleHandler->moduleExists('redirect')) {
      return [];
    }

    $redirects_id = $this->entityTypeManager->getStorage('redirect')->getQuery()->execute();

    /** @var \Drupal\redirect\Entity\Redirect $redirects_load */
    $redirects_load = $this->entityTypeManager->getStorage('redirect')->loadMultiple($redirects_id);

    $redirects = [];
    foreach ($redirects_load as $redirect_load) {
      $source      = $redirect_load->getSourceUrl();
      $destination = $redirect_load->getRedirectUrl()->toString();

      $redirects[$source] = $destination;
    }

    return $redirects;

  }

}
